@@ -716,7 +716,7 @@ int collect_sockets(struct ns_id *ns)
 	close(nl);
 	ns->net.nlsk = -1;
 
-	if (ns->pid == getpid()) {
+	if (err && (ns->pid == getpid())) {
 		/*
 		 * If netns isn't dumped, criu will fail only
 		 * if an unsupported socket will be really dumped.
@@ -727,54 +727,3 @@ int collect_sockets(struct ns_id *ns)
 
 	return err;
 }
-
-static inline char *unknown(u32 val)
-{
-	static char unk[12];
-	snprintf(unk, sizeof(unk), "x%d", val);
-	return unk;
-}
-
-char *skfamily2s(u32 f)
-{
-	if (f == AF_INET)
-		return " inet";
-	else if (f == AF_INET6)
-		return "inet6";
-	else
-		return unknown(f);
-}
-
-char *sktype2s(u32 t)
-{
-	if (t == SOCK_STREAM)
-		return "stream";
-	else if (t == SOCK_DGRAM)
-		return " dgram";
-	else
-		return unknown(t);
-}
-
-char *skproto2s(u32 p)
-{
-	if (p == IPPROTO_UDP)
-		return "udp";
-	else if (p == IPPROTO_UDPLITE)
-		return "udpl";
-	else if (p == IPPROTO_TCP)
-		return "tcp";
-	else
-		return unknown(p);
-}
-
-char *skstate2s(u32 state)
-{
-	if (state == TCP_ESTABLISHED)
-		return " estab";
-	else if (state == TCP_CLOSE)
-		return "closed";
-	else if (state == TCP_LISTEN)
-		return "listen";
-	else
-		return unknown(state);
-}
