@@ -20,6 +20,9 @@
 #ifdef __x86_64__
 # define __NR_fanotify_init	300
 # define __NR_fanotify_mark	301
+#elif defined(__PPC64__)
+# define __NR_fanotify_init	323
+# define __NR_fanotify_mark	324
 #else
 # define __NR_fanotify_init	338
 # define __NR_fanotify_mark	339
